
//////////////////////////////////////////////////////////////
// La communication venant et en direction du master est geree par le hardware
// La communication venant et en direction des octaves suivant est generee par SoftwareSerial

// De maniere alternative on pourrait par le hardware , cela permettrait de laisser le hardware
// s'occuper de la partie intense en communications et le retour vers le master, plus simple, gere
// par le soft

// Tous les messages terminent par '\n'

// Reset général: recu du master: si le message commence par 'R', si c'est une erreur de comm tout
//                est reseté ex ... si le master envoit 'RRRRR' tout est sur d'etre remis à 0, reponse
//                par un "R"+numOctave+"RR\n" vers le master pour signaler
// Assignation du numero d'octave: recu du master: si le message commence par 'S', le numero qui
//                suit est le numero d'octave courant (n) et 'S'+(n+1)+'\n' est transmit et
//                'S'+n+'\n' est renvoye au master
// Demande de l'état des touches: recu du master : precede du numero d'octave, puis 'Q', reponse par
//                un message de la forme numOctave+'T'+etat des touches en hexadecimal dans un string
// Demande de d'allumage de touche: recu du master : precede du numero d'octave, puis 'O' suivi du
//                numero de la touche et du code de la couleur a afficher ('0' pas de couleur), reponse
//                par un message de la forme numOctave+'K'+numero de la touche+code de la couleur
//////////////////////////////////////////////////////////////

#define HARD_SPEED 38400
#define SOFT_SPEED 38400

#include <Wire.h>
#include <TM1650.h>

/*
 * librairies du keypad
 */
#include <Key.h>
#include <Keypad.h>

#include <SoftwareSerial.h>

#define slaveSerial Serial
SoftwareSerial masterSerial( 2, 3 ); // RX, TX

TM1650 displayer;

byte stateReturnPath; // length of messages
byte stateMaster;

char msgReturnPath[ 100 ];  // messages
char msgMaster[ 100 ];
char dbgMaster[ 100 ];
char message[ 6 ];
char lineDisplayer[] = "    ";
char octaveNumber = '-';

const byte numRows = 4; //number of rows on the keypad
const byte numCols = 4; //number of columns on the keypad
char keymap[ numRows ][ numCols ] =
{
    { '1', '2', '3', 'A' },
    { '4', '5', '6', 'B' },
    { '7', '8', '9', 'C' },
    { '*', '0', '#', 'D' }
};
byte rowPins[ numRows ] = { 11, 10, 9, 8 }; //Rows 0 to 3
byte colPins[ numCols ] = { 7, 6, 5, 4 }; //Columns 0 to 3
//initializes an instance of the Keypad class
Keypad myKeypad = Keypad( makeKeymap( keymap ), rowPins, colPins, numRows, numCols );
int message_position;

void debug( char *txt )
{
    octaveNumber = 15;
    masterSerial.write( 'd' );
    masterSerial.write( octaveNumber + 0x30 );
    masterSerial.write( "<=", 2 );
    masterSerial.write( txt );
}

//////////////////////////////////////////////////////////////
void setup( )
{
    slaveSerial.begin( SOFT_SPEED );
    masterSerial.begin( SOFT_SPEED );
    message_position = 1;
    message[ 0 ] = 'K';
    debug( "debugsetup\n" );
    Wire.begin();
    displayer.init();
}

//////////////////////////////////////////////////////////////
void notifyKeypadChanged( )
{
    char keypressed = myKeypad.getKey( );
    if ( keypressed != NO_KEY ) {
        switch ( keypressed ) {
        case '*':
        if ( message_position >= 1 ) {
            message[ message_position ] = 0;
            lineDisplayer[message_position - 1] = ' ';
            message_position--;
        }
        break;
        case '#':
        if ( message_position > 2 ) {
            message[ message_position ] = 0;
            slaveSerial.println( message );
            for ( int n = 1; n < 6; n++ )
                message[ n ] = 0;
            message_position = 1;
            for (int n = 0; n < 4; n++)
            lineDisplayer [n] = ' ';
        }
        break;
        case 'A':
        case 'B':
        case 'C':
        case 'D':
        if ( message_position == 1 ) {
            message[ message_position ] = keypressed;
            lineDisplayer[message_position - 1] = keypressed;
            message_position++;
        }
        break;
        default:
        if ( message_position > 1 && message_position < 5 ) {
            message[ message_position ] = keypressed;
            lineDisplayer[message_position - 1] = keypressed;
            message_position++;
        }
        break;
        }
    }
}

//////////////////////////////////////////////////////////////
void analyseReturnPath( char newIn )
{
    if ( newIn == '$' ) newIn = '\n';
    msgReturnPath[ stateReturnPath++ ] = newIn;
    msgReturnPath[ stateReturnPath ] = 0;
    if ( newIn == '\n' ) { // fin de message
        masterSerial.write( msgReturnPath );
        stateReturnPath = 0;
    }
}

//////////////////////////////////////////////////////////////
void analyseMaster( char newIn )
{
    if ( newIn == '$' ) newIn = '\n';


    msgMaster[ stateMaster++ ] = newIn;
    msgMaster[ stateMaster ] = 0;

    if ( newIn == '\n' ) { // fin de message
        slaveSerial.write( msgMaster, stateMaster ); //not talking to me: bypass
        stateMaster = 0;
    }
}

//////////////////////////////////////////////////////////////
void loop( )
{
    if ( masterSerial.available( ) ) {
        analyseMaster( masterSerial.read( ) );
    }

    if ( slaveSerial.available( ) ) {
        analyseReturnPath( slaveSerial.read( ) );
    }
    notifyKeypadChanged( );
    displayer.displayString(lineDisplayer);
}


